using System;
using System.Collections.Generic;
using System.Text;

namespace Labyrinth
{
    class UnionFind<T>
    {
        private UnionFind<T> father;

        private T content;
        /// <summary>
        /// Any data of the type you chose for this UnionFind object
        /// </summary>
        public T Content
        {
            get { return content; }
            set { content = value; }
        }

        /// <summary>
        /// the representative for the set to which this object belongs
        /// </summary>
        public UnionFind<T> Set
        {
            get
            {
                if (this.father == null)
                    return this;
                else
                {
                    this.father = this.father.Set;
                }
                return father;
            }
        }

        public UnionFind<T> Father
        {
            get { return father; }
            set { father = value; }
        }
        
        /// <summary>
        /// Creates a new UnionFind object
        /// </summary>
        /// <param name="content">Any data of the type you chose for this UnionFind object</param>
        /// <param name="father">Can be null if the object is alone in its set, or any object that is supposed to be in the same set</param>
        public UnionFind(T content, UnionFind<T> father)
        {
            this.content = content;
            this.father = father;
        }

        /// <summary>
        /// Joins the sets of this object and another object.
        /// </summary>
        /// <param name="otherObject">the other object</param>
        public void Union(UnionFind<T> otherObject)
        {
            if(this.Set != otherObject.Set) otherObject.Set.Father = this;
        }
    }
}
